/**
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

/**
 **  gcspy_server_state.hpp
 **
 **  The state maintained by the GCspy server
 **/

#ifndef _GCSPY_SERVER_STATE_HPP_

#define _GCSPY_SERVER_STATE_HPP_

#include "gcspy_types.hpp"

class gcspyServerState {
public:
  enum PublicConstants {
    max_events = 20
  };

private:
  int          _pause_req;
  gcspy_bool_t _restart;
  gcspy_bool_t _play_one;
  gcspy_bool_t _shutdown_req;

  gcspy_bool_t _enabled[max_events];
  int          _delays[max_events];
  gcspy_bool_t _paused[max_events];
  int          _periods[max_events];

public:
  gcspyServerState (void);

  void reset_flags (void);
  void reset_event_filters (void);

  inline gcspy_bool_t is_pause_req_set()    { return _pause_req; }
  inline gcspy_bool_t is_restart_set()      { return _restart; }
  inline gcspy_bool_t is_play_one_set()     { return _play_one; }
  inline gcspy_bool_t is_shutdown_req_set() { return _shutdown_req; }

  inline void set_pause_req()      { _pause_req      = 1; }
  inline void set_restart()        { _restart        = 1; }
  inline void set_play_one()       { _play_one       = 1; }
  inline void set_shutdown_req()   { _shutdown_req   = 1; }

  inline void unset_pause_req()    { _pause_req      = 0; }
  inline void unset_restart()      { _restart        = 0; }
  inline void unset_play_one()     { _play_one       = 0; }
  inline void unset_shutdown_req() { _shutdown_req   = 0; }

  inline gcspy_bool_t *get_enabled_array()       { return _enabled; }
  inline int          *get_delays_array()        { return _delays; }
  inline gcspy_bool_t *get_paused_array()        { return _paused; }
  inline int          *get_periods_array()       { return _periods; }

  inline gcspy_bool_t is_enabled (gcspy_index_t index) {
    return _enabled[index];
  }

  inline int get_delay (gcspy_index_t index) {
    return _delays[index];
  }

  inline gcspy_bool_t is_paused (gcspy_index_t index) {
    return _paused[index];
  }

  inline int get_period (gcspy_index_t index) {
    return _periods[index];
  }
};

#endif //_GCSPY_SERVER_STATE_HPP_
