/**
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

/**
 **  gcspy_timer.cpp
 **
 **  Timer facilities
 **/

#include <time.h>
#include <sys/time.h>
#ifdef _SOLARIS_
#include <unistd.h>
#endif //_SOLARIS_

#include "gcspy_timer.hpp"

void
gcspyTimer::wait (int ms) {
#ifdef _SOLARIS_
  {
    hrtime_t timer;
    timer = gethrtime();
    while ((gethrtime() - timer) < (long long) ms * 1000000) {
      yield();
    }
  }
#endif //_SOLARIS_

#ifdef _LINUX_
  {
    struct timespec _interval;
    struct timespec _spec;

    time_t secs = ms / 1000;
    long nanos = (ms % 1000) * (1000 * 1000);

    _interval.tv_sec  = secs;           /* num of seconds     */
    _interval.tv_nsec = nanos;          /* num of nanoseconds */

    nanosleep(&_interval, &_spec);
  }
#endif //_LINUX_
}

/* time in milliseconds */
long long
gcspyTimer::get_time () {
#ifdef _SOLARIS_
  {
    long long ret = (long long) gethrtime() / (long long) 1000000;
    return ret;
  }
#endif //_SOLARIS_

#ifdef _LINUX_
  {
    struct timeval tv;
    struct timezone tz;
    long long diff;

    gettimeofday(&tv, &tz);
    diff = ((long long) tv.tv_sec) * 1000 + ((long long) tv.tv_usec) / 1000;

    return diff;
  }
#endif //_LINUX_
}
