/**
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

/**
 **  gcspy_utils.cpp
 **
 **  Misc Utilities
 **/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gcspy_utils.hpp"

#define gcspy_set_time_date_vars(_time_) \
  char _tmp[128]; \
  char *time; \
  char *day_month; \
  char *year; \
\
  strcpy(_tmp, ctime(&_time_)); \
  _tmp[10] = '\0'; \
  _tmp[19] = '\0'; \
  _tmp[24] = '\0'; \
  time = _tmp+11; \
  day_month = _tmp; \
  year = _tmp+20

void
gcspyUtils::raise_error (const char *file,
			 int line, 
			 const char *messg) {
  printf("## GCspy Error  (%s, line %d)\n", file, line);
  printf("## %s\n", messg);

  exit(-1);
}

char *
gcspyTimeDate::format_time (char *buffer) {
  gcspy_set_time_date_vars(_time);
  strcpy(buffer, time);
  return buffer;
}

char *
gcspyTimeDate::format_date (char *buffer) {
  gcspy_set_time_date_vars(_time);
  sprintf(buffer, "%s, %s", day_month, year);
  return buffer;
}

char *
gcspyTimeDate::format_time_date (char *buffer) {
  gcspy_set_time_date_vars(_time);
  char tmp1[64], tmp2[64];
  sprintf(buffer, "%s, %s",
	  format_time(tmp1), format_date(tmp2));
  return buffer;
}

char *
gcspyUtils::format_size (gcspy_size_t size, char *buffer) {
  char tmp[64];
  if (size < megabyte)
    sprintf(buffer, "%1.1lfKB (%s bytes)",
	    (double) size / kilobyte_d,
	    format_large_number(size, tmp));
  else
    sprintf(buffer, "%1.1lfMB (%s bytes)",
	    (double) size / megabyte_d,
	    format_large_number(size, tmp));
  return buffer;
}

char *
gcspyUtils::format_large_number (unsigned int num, char *buffer) {
  sprintf(buffer, "%u", num);
  int curr = strlen(buffer) - 3;
  while (curr > 0) {
    memmove(&buffer[curr+1], &buffer[curr], strlen(buffer) - curr+1);
    buffer[curr] = ',';

    curr -= 3;
  }
  return buffer;
}
