package gcspy.vis;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * {@link Thread} that notifies listeners upon termination.
 * 
 * @author Peter Hofer
 * 
 */
public abstract class EnhancedThread extends Thread {

	private Logger logger = Logger.getLogger(EnhancedThread.class.getName());

	private List<EnhancedThreadListener> listeners = null;

	public EnhancedThread() {
	}

	/**
	 * Perform work. Subclasses should override this method.
	 */
	protected void execute() {
	}

	@Override
	public final void run() {
		try {
			execute();
		} catch (Throwable t) {
			logger.log(Level.FINE, "Thread execution terminated with exception", t);
		}

		fireCompletionListeners();
		listeners = null; // clear references
	}

	public void addCompletionListener(EnhancedThreadListener l) {
		if (l == null)
			throw new NullPointerException("listener");

		if (listeners == null) {
			listeners = new ArrayList<EnhancedThreadListener>();
		}
		listeners.add(l);
	}

	public void removeCompletionListener(EnhancedThreadListener l) {
		if (listeners != null) {
			listeners.remove(l);
		}
	}

	protected void fireCompletionListeners() {
		if (listeners != null) {
			for (EnhancedThreadListener l : listeners) {
				l.threadCompleted(this);
			}
		}
	}

}
