/**
 ** $Id: EventCountTableModel.java 28 2005-06-20 13:13:35Z rej $
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

package gcspy.vis;

import gcspy.interpreter.Events;
import gcspy.interpreter.TraceData;

import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

/**
 * Model for a {@link JTable} displaying event counts.
 * 
 * @author Tony Printezis
 * @author Peter Hofer
 */
class EventCountTableModel extends AbstractTableModel {

	private String columnNames[] = { "Event", "Counter" };

	private Object data[][] = null;
	private int rows;

	public int getColumnCount() {
		return columnNames.length;
	}

	public int getRowCount() {
		return rows;
	}

	public String getColumnName(int col) {
		return columnNames[col];
	}

	public Object getValueAt(int row, int col) {
		return data[row][col];
	}

	/**
	 * Get the class of this column
	 * 
	 * @return the class of this column
	 */
	public Class<?> getColumnClass(int c) {
		return getValueAt(0, c).getClass();
	}

	public boolean isCellEditable(int row, int col) {
		// No cells are editable
		return false;
	}

	/**
	 * Set the table data
	 * 
	 * @param count
	 *            The new event counts
	 */
	public void setData(int count[]) {
		for (int i = 0; i < rows; ++i) {
			data[i][1] = count[i];
		}
		fireTableDataChanged();
	}

	/**
	 * Sets the trace data displayed by a table backed by this model.
	 * 
	 * @param tdata
	 *            Trace data to display (may be <code>null</code> to clear
	 *            view).
	 */
	public void setTraceData(TraceData tdata) {
		if (tdata != null) {
			Events events = tdata.getEvents();
			rows = events.getNum();
			data = new Object[rows][];

			for (int i = 0; i < rows; ++i) {
				data[i] = new Object[columnNames.length];
				data[i][0] = events.getName(i);
				data[i][1] = 0;
			}
		} else {
			rows = 0;
			data = null;
		}

		fireTableDataChanged();
	}

	/**
	 * Constructor.
	 */
	EventCountTableModel() {
	}

}
