package gcspy.vis;

/**
 * Interface for viewers for properties of type {@code T}.
 * 
 * A property view has only a single provider of properties at a time. When new
 * properties are set by a different object, it becomes the new provider.
 * Providers and other can subscribe for provider change notifications.
 * 
 * @author Peter Hofer
 * 
 * @param <T>
 *            Type of properties displayed.
 */
public interface PropertyView<T> {

	/**
	 * Sets the specified object as property data and the .
	 * 
	 * @param provider
	 *            Property source (may be <code>null</code>).
	 * @param properties
	 *            Property data.
	 */
	void setProperties(Object provider, T properties);

	/**
	 * Adds a listener to be notified when the property source changes.
	 * 
	 * @param listener
	 *            The listener object to notify.
	 */
	public void addPropertySourceListener(PropertySourceListener<T> listener);
}
