package gcspy.vis;

import java.awt.Color;
import java.awt.Component;

import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableCellRenderer;

/**
 * {@link TableCellRenderer} rendering cell values using a {@link JSpinner}.
 * 
 * @author Peter Hofer
 */
public class SpinnerTableCellRenderer extends JSpinner implements TableCellRenderer {

	private static final Border NO_FOCUS_BORDER = new EmptyBorder(1, 1, 1, 1);

	private Color unselectedForeground = null;
	private Color unselectedBackground = null;

	/**
	 * Constructrs a new {@link TableCellRenderer}.
	 */
	public SpinnerTableCellRenderer() {
	}

	@Override
	public void setForeground(Color fg) {
		unselectedForeground = fg;
		super.setForeground(fg);
	}

	@Override
	public void setBackground(Color bg) {
		unselectedBackground = bg;
		super.setBackground(bg);
	}

	@Override
	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus,
			int row, int column) {

		// Most of the code below is from DefaultTableCellRenderer

		Color fg = null;
		Color bg = null;

		JTable.DropLocation dropLocation = table.getDropLocation();
		if (dropLocation != null && !dropLocation.isInsertRow() && !dropLocation.isInsertColumn()
				&& dropLocation.getRow() == row && dropLocation.getColumn() == column) {

			fg = UIManager.getColor("Table.dropCellForeground");
			bg = UIManager.getColor("Table.dropCellBackground");

			isSelected = true;
		}

		if (isSelected) {
			super.setForeground(fg == null ? table.getSelectionForeground() : fg);
			super.setBackground(bg == null ? table.getSelectionBackground() : bg);
		} else {
			super.setForeground(unselectedForeground != null ? unselectedForeground : table.getForeground());
			super.setBackground(unselectedBackground != null ? unselectedBackground : table.getBackground());
		}

		setFont(table.getFont());

		if (hasFocus) {
			Border border = null;
			if (isSelected) {
				border = UIManager.getBorder("Table.focusSelectedCellHighlightBorder");
			}
			if (border == null) {
				border = UIManager.getBorder("Table.focusCellHighlightBorder");
			}
			setBorder(border);

			if (!isSelected && table.isCellEditable(row, column)) {
				Color col;
				col = UIManager.getColor("Table.focusCellForeground");
				if (col != null) {
					super.setForeground(col);
				}
				col = UIManager.getColor("Table.focusCellBackground");
				if (col != null) {
					super.setBackground(col);
				}
			}
		} else {
			setBorder(NO_FOCUS_BORDER);
		}

		setValue(value);
		return this;
	}

}
