/**
 ** $Id: ImageUtils.java 22 2005-06-14 05:17:45Z rej $
 *Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

package gcspy.vis.utils;

import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

/**
 * Utilities for image manipulation
 * 
 * @author Tony Printezis
 */
public class ImageUtils {

	/**
	 * Crop image
	 * 
	 * @param image
	 *            The image
	 * @param x
	 *            x-cooordinate of the image origin
	 * @param y
	 *            y-cooordinate of the image origin
	 * @param width
	 *            The width to crop to
	 * @param height
	 *            The height to crop to
	 * @return the cropped image
	 */
	static public BufferedImage cropImage(BufferedImage image, int x, int y, int width, int height) {
		BufferedImage cropped = new BufferedImage(width, height, image.getType());
		Graphics g = cropped.getGraphics();
		g.drawImage(image, x, y, null);
		g.dispose();
		return cropped;
	}

	/**
	 * Export a {@link RenderedImage} instance to a PNG file.
	 * 
	 * @param image
	 *            The image to export.
	 * @param file
	 *            The file to write to.
	 * @param overwrite
	 *            <code>true</code> if the file should be overwritten if it
	 *            already exists.
	 * @return <code>true</code> if the file was written, <code>false</code>
	 *         otherwise.
	 * @throws IOException
	 *             if an error occurs when writing.
	 */
	public static boolean exportToPNG(RenderedImage image, File file, boolean overwrite) throws IOException {
		if (overwrite || !file.exists()) {
			return ImageIO.write(image, "png", file);
		}
		return false;
	}
}
